#pragma GCC diagnostic ignored "-Wdeprecated-declarations"

/* 	Copyright John D. "smeezekitty" 2017
	
	FreeNBTV is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    FreeNBTV is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with FreeNBTV.  If not, see <http://www.gnu.org/licenses/>.
*/

/* This file contains generic GUI functions */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <gtk/gtk.h>
#include <SDL/SDL.h>
#include <SDL/SDL_image.h>
#include "nbtv_format.h"
#include "util.h"
#include "gui.h"
#include "input.h"
#include "preview.h"

SDL_Surface *screen;

extern const char *y_u_no_guy[];

GtkWidget *create_number_box(int width){
	GtkWidget *box;
	box = gtk_entry_new();
	gtk_entry_set_max_length(GTK_ENTRY(box), width);
	gtk_entry_set_width_chars(GTK_ENTRY(box), width);
//	gtk_entry_set_input_purpose(GTK_ENTRY(box), width);
	return box;
}


void destroy(GtkWidget *w, gpointer d){
	SDL_Quit();
	exit(0);
}
GtkWidget *put_label(GtkWidget *grid, char *text, int x, int y){
	GtkWidget *L = gtk_label_new(text);
	//gtk_label_set_xalign(GTK_LABEL(L), 0.2);
	gtk_widget_set_margin_left(L, 0);
	gtk_grid_attach(GTK_GRID(grid), L, x, y, 1, 1);
	return L;
}
GtkWidget *put_label_colored(GtkWidget *grid, int r, int g, int b, char *text, int x, int y){
	GtkWidget *L = gtk_label_new(NULL);
	//gtk_label_set_xalign(GTK_LABEL(L), 0.2);
	gtk_widget_set_margin_left(L, 0);
	gtk_label_set_markup(GTK_LABEL(L), g_markup_printf_escaped("<span color=\"#%02X%02X%02X\">%s</span>", r, g, b, text));
	gtk_grid_attach(GTK_GRID(grid), L, x, y, 1, 1);
	return L;
}
int gui_tick(){
	SDL_Event ev;
	while(gtk_events_pending()){
		gtk_main_iteration();
	}
	while(SDL_PollEvent(&ev)){
		switch(ev.type){
			case SDL_QUIT:
				return 0;
				break;
			case SDL_MOUSEBUTTONDOWN:
				if(ev.button.button == SDL_BUTTON_LEFT)	//User clicked inside the SDL window. See if we can move the input coordinants 
					handle_mouse_repos(&input, ev.button.x, ev.button.y); 
				break;
		}
	}
	return 1;
}
/*Lowest log level*/
void debug(const char *format, ...){
	char buffer[128];
	va_list va;
	if(log_level < LOG_DEBUG) return;
	va_start(va, format);
	sprintf(buffer, "Debug: %s%s", format, format[strlen(format)-1]=='\n'? "": "\n");
	vprintf(buffer, va);
	va_end(va);
}
void info(const char *format, ...){
	char buffer[128];
	va_list va;
	if(log_level < LOG_INFO) return;
	va_start(va, format);
	sprintf(buffer, "Info: %s%s", format, format[strlen(format)-1]=='\n'? "": "\n");
	vprintf(buffer, va);
	va_end(va);
}
void warn(const char *format, ...){
	char buffer[128];
	va_list va;
	if(log_level < LOG_WARN) return;
	va_start(va, format);
	sprintf(buffer, "Warning: %s%s", format, format[strlen(format)-1]=='\n'? "": "\n");
	vprintf(buffer, va);
	va_end(va);
	fflush(stdout);
}
/*Highest log level*/
void error(const char *format, ...){
	char buffer[128];
	GtkWidget *dialog;
	va_list va;
	va_start(va, format);
	vsprintf(buffer, format, va);
	printf("Error: %s%s", buffer, buffer[strlen(buffer)]=='\n'? "": "\n");
	dialog = gtk_dialog_new_with_buttons("Error", NULL, GTK_DIALOG_MODAL, "_OK", GTK_RESPONSE_ACCEPT, NULL);
	gtk_box_pack_end(GTK_BOX(gtk_dialog_get_content_area(GTK_DIALOG(dialog))), gtk_label_new(buffer), TRUE, TRUE, 0);
	gtk_widget_show_all(dialog);
	gtk_dialog_run(GTK_DIALOG(dialog));
	gtk_widget_destroy(dialog);
	va_end(va);
	fflush(stdout);
}

/*Memetic goodness*/
void y_u_no_error(){
	GdkPixbuf *pb;
	GtkWidget *dialog;
	pb = gdk_pixbuf_new_from_xpm_data(y_u_no_guy);
	dialog = gtk_dialog_new_with_buttons("Error", NULL, GTK_DIALOG_MODAL, "_OK", GTK_RESPONSE_ACCEPT, NULL);
	gtk_box_pack_end(GTK_BOX(gtk_dialog_get_content_area(GTK_DIALOG(dialog))), gtk_image_new_from_pixbuf(pb), TRUE, TRUE, 0);
	gtk_widget_show_all(dialog);
	gtk_dialog_run(GTK_DIALOG(dialog));
	gtk_widget_destroy(dialog);
}
void gui_init(){
	load_formats();
	create_input_window();
	create_format_window();
	gui_update_format(nbtv_fmt);
	init_input();
	gui_format_connect_callbacks();
	gui_create_output_window();
	gui_update_input();
}