#pragma GCC diagnostic ignored "-Wdeprecated-declarations"

/* 	Copyright John D. "smeezekitty" 2017
	
	FreeNBTV is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    FreeNBTV is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with FreeNBTV.  If not, see <http://www.gnu.org/licenses/>.
*/

/* This file is the input layer. It handles loading and prescaling of the image or video and sending it to be converted to NBTV*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include <gtk/gtk.h>
#include <SDL/SDL.h>
#include <SDL/SDL_image.h>
#include <libavcodec/avcodec.h>
#include <libavformat/avformat.h>
#include <libswresample/swresample.h>
#include <libswscale/swscale.h>
#include <gdk-pixbuf/gdk-pixbuf.h>
#include "nbtv_format.h"
#include "input.h"
#include "gui.h"
#include "nbtv_gen.h"
#include "util.h"

AVFormatContext *fmt_context = NULL;
AVCodecContext *codec_context, *audio_codec_context;
AVCodec *decoder, *audio_decoder;
AVFrame *av_frame = NULL;
AVFrame *af_frame = NULL;
AVFrame *av_frame_scaled = NULL;
AVPacket packet;
AVPacket audio_packets[65536];
int ap_ptr;
int audio_packet_pointer;
struct SwsContext *scaler = NULL;
struct SwrContext *resampler = NULL;
int video_stream, audio_stream;
SDL_Surface *image;

struct _input input;
double input_timestamp;
unsigned int delay_value;
char kill_thread;

//extern int fetch_audio_sample_block(int desired_length, long timestamp, float *buffer, struct _output *op);


/*Perform a blit between two SDL surfaces with primitive software scaling*/
void scale_blit(SDL_Surface *dest, SDL_Rect *dr, SDL_Surface *src, int sx, int sy, int w, int h){
	int dx, dy, dw, dh;
	uint8_t *dpixels,*spixels;
	if(!src || !dest) return;
	if(!dr){
		dx = 0;
		dy = 0;
		dw = dest->w;
		dh = dest->h;
	} else {
		dx = dr->x;
		dy = dr->y;
		dw = dr->w;
		dh = dr->h;
	}
	if(sx > src->w || sy > src->h){
		return;
	}
	if(dx > dest->w || dy > src->h){
		return;
	}
	int base_addr_dest, base_addr_src;
	if(w+sx > src->w){w = src->w-sx;}
	if(h+sy > src->h){h = src->h-sy;}
	if(dw+dx > dest->w){dw = dest->w-dx;}
	if(dh+dy > dest->h){dh = dest->h-dy;}
	debug("Scale_blit(): w=%d h=%d sx = %d sy = %d, dw = %d dh = %d dx = %d dy = %d\n", w, h, sx, sy, dw, dh, dx, dy);
	dpixels = (uint8_t *)dest->pixels;
	spixels = (uint8_t *)src->pixels;
	double spx=0,spy=0;
	double sppdpx=(double)w/(double)dw;
	double sppdpy=(double)h/(double)dh;
	int s;
	debug("sppdpx=%f sppdpy = %f\n", sppdpx, sppdpy);
	int x,y;
	for(x = dx;x < dx+dw;x++){
		for(y = dy;y < dy+dh;y++){
				base_addr_dest = (y * dest->pitch) + x*dest->format->BytesPerPixel;
				base_addr_src = ((sy + (int)floor(spy)) * src->pitch) + (sx + (int)floor(spx))*src->format->BytesPerPixel;
				dpixels[base_addr_dest] = spixels[base_addr_src];
				dpixels[base_addr_dest + 1] = spixels[base_addr_src + 1];
				dpixels[base_addr_dest + 2] = spixels[base_addr_src + 2];
				spy += sppdpy;
			//printf("dx = %d dy = %d spx=%0.1f spy=%0.1f\n", x, y, spx, spy);
		}
		spy = 0;
		spx += sppdpx;
	}
}
/*Takes a rectangular shaped chunk and scales it to the proper size for conversion to NBTV*/
void rectangular_window(SDL_Surface *dest, struct NBTV_format *fmt, SDL_Surface *src, struct _input *i){
	int dx, dy, dw, dh;
	int sx,sy,w,h;
	uint8_t *dpixels,*spixels;
	if(!src || !dest) return;
	dx = dy = 0;
	dw = fmt->imgW;
	dh = fmt->imgH;
	if(i->rotation_angle & 1){
		h = i->w*i->XRR;
		w = i->h2*i->YRR;
		sy = i->y*i->XRR - h/2;
		sx = i->x*i->XRR - w/2;
	} else {
		w = i->w*i->XRR;
		h = i->h2*i->YRR;		
		sx = i->x*i->XRR - w/2;
		sy = i->y*i->YRR - h/2;;
	}
	if(sx > src->w || sy > src->h){
		return;
	}
	if(dx > dest->w || dy > src->h){
		return;
	}
	int base_addr_dest, base_addr_src;
	if(w+sx/2 > src->w){w = src->w-sx/2;}
	if(h+sy/2 > src->h){h = src->h-sy/2;}
	if(dw+dx > dest->w){dw = dest->w-dx;}
	if(dh+dy > dest->h){dh = dest->h-dy;}
	//printf("Rectangular window(): w=%d h=%d sx = %d sy = %d, dw = %d dh = %d dx = %d dy = %d\n", w, h, sx, sy, dw, dh, dx, dy);
	dpixels = (uint8_t *)dest->pixels;
	spixels = (uint8_t *)src->pixels;
	int s;
	double spx=0,spy=0;
	double sppdpx=(double)w/(double)dw;
	double sppdpy=(double)h/(double)dh;
	debug("sppdpx=%f sppdpy = %f\n", sppdpx, sppdpy);
	int x,y;
	int R,G,B;
	for(x = dx;x < dx+dw;x++){
		for(y = dy;y < dy+dh;y++){
			base_addr_dest = (y * dest->pitch) + x*dest->format->BytesPerPixel;
			/*base_addr_src = ((sy + (int)floor(spy)) * src->pitch) + (sx + (int)floor(spx))*src->format->BytesPerPixel;
			R = spixels[base_addr_src + 2];
			G = spixels[base_addr_src + 1];
			B = spixels[base_addr_src];*/
			R = G = B = 0;
			int cv = ceil(sppdpy);
			/*If there is more than one input pixel per output pixel, combine and average them*/
			for(s = 0;s < cv;s++){
				//printf("s=%d\n", s);
				base_addr_src = ((sy + (int)floor(spy+s)) * src->pitch) + (sx + (int)floor(spx))*src->format->BytesPerPixel;
				R += spixels[base_addr_src + 2]/cv;
				G += spixels[base_addr_src + 1]/cv;
				B += spixels[base_addr_src]/cv;
			}
			//Brightness + Contrast
			R = ((double)(fmt->contrast+100)/100.0)*(R - 128.0) + 128.0 + fmt->brightness;
			G = ((double)(fmt->contrast+100)/100.0)*(G - 128.0) + 128.0 + fmt->brightness;
			B = ((double)(fmt->contrast+100)/100.0)*(B - 128.0) + 128.0 + fmt->brightness;
			if(R > 255){R = 255;}
			if(G > 255){G = 255;}
			if(B > 255){B = 255;}
			if(R < 0){R = 0;}
			if(G < 0){G = 0;}
			if(B < 0){B = 0;}
			dpixels[base_addr_dest] = B;
			dpixels[base_addr_dest + 1] = G;
			dpixels[base_addr_dest + 2] = R;
			spy += sppdpy;
			//printf("dx = %d dy = %d spx=%0.1f spy=%0.1f\n", x, y, spx, spy);
		}
		spy = 0;
		spx += sppdpx;
	}
}	
/*The following 3 functions take a trapezoidal shaped chunk of the input and convert it to the appropriate sized rectangle by scaling*/
void trapizoidal_window_90(SDL_Surface *dest, struct NBTV_format *fmt, SDL_Surface *src, struct _input *i){
	int dx, dy, dw, dh;
	int sx,sy,w,h;
	uint8_t *dpixels,*spixels;
	if(!src || !dest) return;
	dx = dy = 0;
	dw = fmt->imgW;
	dh = fmt->imgH;
	h = i->w*i->XRR;
	//w = i->h2*i->YRR;
	sy = i->y*i->XRR - h/2;
	sx = (i->x - i->h2/2)*i->XRR;
	if(sx > src->w || sy > src->h){
		return;
	}
	if(dx > dest->w || dy > src->h){
		return;
	}
	int base_addr_dest, base_addr_src;
	if(w+sx/2 > src->w){w = src->w-sx/2;}
	if(h+sy/2 > src->h){h = src->h-sy/2;}
	if(dw+dx > dest->w){dw = dest->w-dx;}
	if(dh+dy > dest->h){dh = dest->h-dy;}
	//printf("Rectangular window(): w=%d h=%d sx = %d sy = %d, dw = %d dh = %d dx = %d dy = %d\n", w, h, sx, sy, dw, dh, dx, dy);
	dpixels = (uint8_t *)dest->pixels;
	spixels = (uint8_t *)src->pixels;
	int s;
	double spx=0,spy=0;
	double sppdpy=(double)h/(double)dh, sppdpx;
	int x,y;
	int R,G,B;
	for(y = dy;y < dy+dh;y++){
		w = (i->h1 + ((double)(i->h2 - i->h1) * (double)(y - dy)/(double)dh)) * i->YRR;
			double sppdpx=(double)w/(double)dw;
			debug("sppdpx=%f sppdpy = %f\n", sppdpx, sppdpy);
			sx = i->x*i->XRR - w/2;
		for(x = dx;x < dx+dw;x++){		

			base_addr_dest = (y * dest->pitch) + x*dest->format->BytesPerPixel;
			/*base_addr_src = ((sy + (int)floor(spy)) * src->pitch) + (sx + (int)floor(spx))*src->format->BytesPerPixel;
			R = spixels[base_addr_src + 2];
			G = spixels[base_addr_src + 1];
			B = spixels[base_addr_src];*/
			R = G = B = 0;
			for(s = 0;s < ceil(sppdpy);s++){
				//printf("s=%d\n", s);
				base_addr_src = ((sy + (int)floor(spy+s)) * src->pitch) + (sx + (int)floor(spx))*src->format->BytesPerPixel;
				R += spixels[base_addr_src + 2]/ceil(sppdpy);
				G += spixels[base_addr_src + 1]/ceil(sppdpy);
				B += spixels[base_addr_src]/ceil(sppdpy);
			}
			R = ((double)(fmt->contrast+100)/100.0)*(R - 128.0) + 128.0 + fmt->brightness;
			G = ((double)(fmt->contrast+100)/100.0)*(G - 128.0) + 128.0 + fmt->brightness;
			B = ((double)(fmt->contrast+100)/100.0)*(B - 128.0) + 128.0 + fmt->brightness;
			if(R > 255){R = 255;}
			if(G > 255){G = 255;}
			if(B > 255){B = 255;}
			if(R < 0){R = 0;}
			if(G < 0){G = 0;}
			if(B < 0){B = 0;}
			dpixels[base_addr_dest] = B;
			dpixels[base_addr_dest + 1] = G;
			dpixels[base_addr_dest + 2] = R;
			debug("spx=%0.1f spy=%0.1f\n", spx, spy);
			spx += sppdpx;
			//printf("dx = %d dy = %d spx=%0.1f spy=%0.1f\n", x, y, spx, spy);
		}
		spx = 0;
		spy += sppdpy;
	}
}			
void trapizoidal_window_270(SDL_Surface *dest, struct NBTV_format *fmt, SDL_Surface *src, struct _input *i){
	int dx, dy, dw, dh;
	int sx,sy,w,h;
	uint8_t *dpixels,*spixels;
	if(!src || !dest) return;
	dx = dy = 0;
	dw = fmt->imgW;
	dh = fmt->imgH;
	h = i->w*i->XRR;
	//w = i->h2*i->YRR;
	sy = i->y*i->XRR - h/2;
	sx = (i->x - i->h1/2)*i->XRR;
	if(sx > src->w || sy > src->h){
		return;
	}
	if(dx > dest->w || dy > src->h){
		return;
	}
	int base_addr_dest, base_addr_src;
	if(w+sx/2 > src->w){w = src->w-sx/2;}
	if(h+sy/2 > src->h){h = src->h-sy/2;}
	if(dw+dx > dest->w){dw = dest->w-dx;}
	if(dh+dy > dest->h){dh = dest->h-dy;}
	//printf("Rectangular window(): w=%d h=%d sx = %d sy = %d, dw = %d dh = %d dx = %d dy = %d\n", w, h, sx, sy, dw, dh, dx, dy);
	dpixels = (uint8_t *)dest->pixels;
	spixels = (uint8_t *)src->pixels;
	int s;
	double spx=0,spy=0;
	double sppdpy=(double)h/(double)dh, sppdpx;
	int x,y;
	int R,G,B;
	for(y = dy;y < dy+dh;y++){
		w = (i->h2 + ((double)(i->h1 - i->h2) * (double)(y - dy)/(double)dh)) * i->YRR;
			double sppdpx=(double)w/(double)dw;
			debug("sppdpx=%f sppdpy = %f\n", sppdpx, sppdpy);
			sx = i->x*i->XRR - w/2;
		for(x = dx;x < dx+dw;x++){		

			base_addr_dest = (y * dest->pitch) + x*dest->format->BytesPerPixel;
			/*base_addr_src = ((sy + (int)floor(spy)) * src->pitch) + (sx + (int)floor(spx))*src->format->BytesPerPixel;
			R = spixels[base_addr_src + 2];
			G = spixels[base_addr_src + 1];
			B = spixels[base_addr_src];*/
			R = G = B = 0;
			for(s = 0;s < ceil(sppdpy);s++){
				//printf("s=%d\n", s);
				base_addr_src = ((sy + (int)floor(spy+s)) * src->pitch) + (sx + (int)floor(spx))*src->format->BytesPerPixel;
				R += spixels[base_addr_src + 2]/ceil(sppdpy);
				G += spixels[base_addr_src + 1]/ceil(sppdpy);
				B += spixels[base_addr_src]/ceil(sppdpy);
			}
			R = ((double)(fmt->contrast+100)/100.0)*(R - 128.0) + 128.0 + fmt->brightness;
			G = ((double)(fmt->contrast+100)/100.0)*(G - 128.0) + 128.0 + fmt->brightness;
			B = ((double)(fmt->contrast+100)/100.0)*(B - 128.0) + 128.0 + fmt->brightness;
			if(R > 255){R = 255;}
			if(G > 255){G = 255;}
			if(B > 255){B = 255;}
			if(R < 0){R = 0;}
			if(G < 0){G = 0;}
			if(B < 0){B = 0;}
			dpixels[base_addr_dest] = B;
			dpixels[base_addr_dest + 1] = G;
			dpixels[base_addr_dest + 2] = R;
			debug("spx=%0.1f spy=%0.1f\n", spx, spy);
			spx += sppdpx;
			//printf("dx = %d dy = %d spx=%0.1f spy=%0.1f\n", x, y, spx, spy);
		}
		spx = 0;
		spy += sppdpy;
	}
}			
void trapizoidal_window(SDL_Surface *dest, struct NBTV_format *fmt, SDL_Surface *src, struct _input *i){
	if(i->rotation_angle == 1){
		trapizoidal_window_90(dest, fmt, src, i);
		return;
	}
	if(i->rotation_angle == 3){
		trapizoidal_window_270(dest, fmt, src, i);
		return;
	}
	int dx, dy, dw, dh;
	int w,h,sx,sy;
	int h1,h2;
	if(i->rotation_angle){
		h1 = i->h2;
		h2 = i->h1;
	} else {
		h1 = i->h1;
		h2 = i->h2;
	}
	w = i->w*i->XRR;
	h = h2*i->YRR;		
	sx = i->x*i->XRR - w/2;
	sy = i->y*i->YRR - h/2;;
	uint8_t *dpixels,*spixels;
	if(!src || !dest) return;
	dx = dy = 0;
	dw = fmt->imgW;
	dh = fmt->imgH;
	if(sx > src->w || sy > src->h){
		return;
	}
	if(dx > dest->w || dy > src->h){
		return;
	}
	int base_addr_dest, base_addr_src;
	if(w+sx/2 > src->w){w = src->w-sx/2;}
	if(h+sy/2 > src->h){h = src->h-sy/2;}
	if(dw+dx > dest->w){dw = dest->w-dx;}
	if(dh+dy > dest->h){dh = dest->h-dy;}
	debug("Trapazoidal window(): w=%d h=%d sx = %d sy = %d, dw = %d dh = %d dx = %d dy = %d\n", w, h, sx, sy, dw, dh, dx, dy);
	dpixels = (uint8_t *)dest->pixels;
	spixels = (uint8_t *)src->pixels;
	int s;
	double spx=0,spy=0;
	double sppdpx=(double)w/(double)dw, sppdpy;
	//printf("sppdpx=%f sppdpy = %f\n", sppdpx, sppdpy);
	int x,y;
	int R,G,B;
	for(x = dx;x < dx+dw;x++){
		h = (h1 + (double)(h2 - h1)*(double)((double)(x-dx)/(double)dw));
		sy = (i->y - h/2) * i->YRR;
		h *= i->YRR;
		if(sy < 0){sy = 0;}
		if(h < 0){h = 0;}
		debug("sh=%d sy=%d\n", h, sy);
		sppdpy=(double)h/(double)dh;
		for(y = dy;y < dy+dh;y++){
			base_addr_dest = (y * dest->pitch) + x*dest->format->BytesPerPixel;
			/*base_addr_src = ((sy + (int)floor(spy)) * src->pitch) + (sx + (int)floor(spx))*src->format->BytesPerPixel;
			R = spixels[base_addr_src + 2];
			G = spixels[base_addr_src + 1];
			B = spixels[base_addr_src];*/
			R = G = B = 0;
			for(s = 0;s < ceil(sppdpy);s++){
				//printf("s=%d\n", s);
				base_addr_src = ((sy + (int)floor(spy+s)) * src->pitch) + (sx + (int)floor(spx))*src->format->BytesPerPixel;
				R += spixels[base_addr_src + 2]/ceil(sppdpy);
				G += spixels[base_addr_src + 1]/ceil(sppdpy);
				B += spixels[base_addr_src]/ceil(sppdpy);
			}
			R = ((double)(fmt->contrast+100)/100.0)*(R - 128.0) + 128.0 + fmt->brightness;
			G = ((double)(fmt->contrast+100)/100.0)*(G - 128.0) + 128.0 + fmt->brightness;
			B = ((double)(fmt->contrast+100)/100.0)*(B - 128.0) + 128.0 + fmt->brightness;
			if(R > 255){R = 255;}
			if(G > 255){G = 255;}
			if(B > 255){B = 255;}
			if(R < 0){R = 0;}
			if(G < 0){G = 0;}
			if(B < 0){B = 0;}
			dpixels[base_addr_dest] = B;
			dpixels[base_addr_dest + 1] = G;
			dpixels[base_addr_dest + 2] = R;
			spy += sppdpy;
			//printf("dx = %d dy = %d spx=%0.1f spy=%0.1f\n", x, y, spx, spy);
		}
		spy = 0;
		spx += sppdpx;
	}
}
/*Set up the defaults*/	
void init_input(){
	input.window_type = 0;
	input.rotation_angle = 0;
	input.lock_aspect_to_screen_format = 1;
	input.input_aspect = 0.667;
	input.x = 0;
	input.y = 0;
	input.w = 32;
	input.h2 = 48;
	input.Xres = 32;
	input.Yres = 48;
}

int fetch_audio_sample_block(int desired_length, double timestamp, float *buffer, struct _output *op){
	int ts = timestamp / delay_value;
	double tb = (double)audio_codec_context->time_base.num*1000.0f / (double)audio_codec_context->time_base.den;
	int i;
	int ticks = 0, t2=0;
	int tl = 0;
	int start;
	int audio_ready=0;
	debug("f_a_s_n fts=%d, tb = %0.1f", ts, tb);
	for(i = 10;i < ap_ptr;i++){
		if(audio_packets[i].pts*tb > timestamp){
			break;
		}
	}
	i--;
	debug("Starting decode at frame %d", i);
	while(desired_length){
		ticks = 0;
		audio_ready = 0;
		while(!audio_ready){
			avcodec_decode_audio4(audio_codec_context, af_frame, &audio_ready,&audio_packets[i++]); 	
			ticks++;
			debug("Ticks=%d", ticks);
			if(ticks > 8 || i>ap_ptr) return tl;
		}
		start = 0;
		/*if((af_frame->pts * tb) < timestamp){
			start = timestamp - (af_frame->pts * tb);
			if(start < 0) start = 0;
			if(start >= af_frame->nb_samples) start = af_frame->nb_samples - 1;
			debug("Setting start to %d", start);
		
		}*/
		debug("ts=%d (%d) pts=%d len=%d start=%d", (int)(af_frame->best_effort_timestamp*tb), (int)(af_frame->best_effort_timestamp*delay_value*tb), (int)(af_frame->pts*tb), (int)(af_frame->nb_samples), (int)start);
		if(desired_length < af_frame->nb_samples) warn("Sample underrun");
		if(start > af_frame->nb_samples) continue;
		//start *= af_frame->channels * 2;
		int len = min(desired_length, af_frame->nb_samples - start);
		debug("len=%d", len);
		swr_convert(resampler, &buffer, len, &af_frame->data, len);
		desired_length -= len;
		buffer += len;
		tl += len;
		if(t2++ > 64) return tl;
	}
	return tl;
}/*
int fetch_audio_sample_block(int desired_length, long timestamp, float *buffer, struct _output *op){
	static int packet;
	static int sample;
	static int16_t sbuffer[8192];
	static int scount;
	uint16_t *in;
	int ticks;
	int audio_ready = 0;
	if(timestamp == 0){
		af_frame->nb_samples = 0;
	}
	while(desired_length){
		if(af_frame->nb_samples == sample){
			audio_ready = 0;
			while(!audio_ready){
				avcodec_decode_audio4(audio_codec_context, af_frame, &audio_ready,&audio_packets[packet++]); 	
				ticks++;
				debug("Ticks=%d", ticks);
				if(ticks > 8 || packet>ap_ptr) return 0;
			}
			sample = 0;
			scount = af_frame->nb_samples;
			memcpy(sbuffer, af_frame->data[0], min(8192, scount * 2));
		}
		if(scount == 0){
			scount = af_frame->nb_samples;
		}
		int len = min(desired_length, scount - sample);
		if(len < 1) return 0;
		if(sample > scount)
			error("Shits fucked");
		//swr_convert(resampler, &buffer, len, &sbuffer + sample, len);
		debug("len=%d", len);
		int i = 0;
		for(i = 0;i < len;i++){
			buffer[i] = (float)sbuffer[i + sample] / 32768.0f;
		}
		buffer += len;
		desired_length -= len;
		sample += len;
	}
	return 0;
	
}*/
/*
double get_rect_width(){
	if(input.window_type == 2){
		return input.r2 - input.r1;
	}
	return input.width;
}
double get_rect_height(){
	if(input.window_type == 1){
		return (input.p1.base1 + input.p2.base2)/2.0;
	}
	if(input.window_type == 2){
		return ((input.p1.outer_radius + input.p2.inner_radius) * 3.1416) * (input.p3.angle / 360.0);
	}
	return input.p2.height;
}*/
/*Finds the video stream and returns its indice or -1 if it can't find one*/
int find_video_stream(AVFormatContext *f){
	int i = 0;
	for(i = 0;i < f->nb_streams; i++){
		if(f->streams[i]->codec->codec_type==AVMEDIA_TYPE_VIDEO) {
			return i;
		}
	}
	return -1;
}
/*Ditto for the audio stream*/
int find_audio_stream(AVFormatContext *f){
	int i = 0;
	
	for(i = 0;i < f->nb_streams; i++){
		if(f->streams[i]->codec->codec_type==AVMEDIA_TYPE_AUDIO) {
			return i;
		}
	}
	return -1;
}
/*Grabs, decodes and scales frames one at a time so they can be sent to the NBTV generator. 
This is currently asyncronous so screen tearing is a problem*/
int video_thread(void *data){
	static int i;
	int frame_ready;
	int audio_ready;
	int out_stride = input.sxres * 3;
	do {
		for(i = 0;av_read_frame(fmt_context, &packet) >= 0;){ //Continue until we reach the last frame
			if(kill_thread){ //Or stop
				debug("Returning after KT\n");
				return 0;
			}
			if(packet.stream_index == video_stream){
				i++;
				avcodec_decode_video2(codec_context, av_frame, &frame_ready, &packet); //Decode the actual frame
				if(frame_ready){ //Is it a complete frame?
					debug("Frame %d\n", i);
					/*Good! Scale the frame using ffmpeg's scaler so the other layers don't have to. It has better interpolation anyway*/
					sws_scale(scaler, (void*)av_frame->data, av_frame->linesize, 0, av_frame->height, (void*)&image->pixels, (void*)&out_stride);
					frame_changed = 1; //Signal the other layers there is a new frame
				}
				//memcpy(screen->pixels, (void*)out, 320*240*3);
				/*This is a very approximate way to make it run at the correct frame rate. Realistically, it isn't that great because
				SDL_Delay isn't the precise, varies by platform and doesn't accept fractional milliseconds. We should correct for this
				but that will probably happen when audio is added. With video only, it is good enough*/
				SDL_Delay(delay_value); 
				input_timestamp += delay_value;
				/*Keep tract of the timestamp of playback*/
			}
			if(audio_stream != -1 && audio_codec_context && packet.stream_index == audio_stream && ap_ptr < 65535){
				//avcodec_decode_audio4(audio_codec_context, af_frame, &audio_ready, &packet);
				//if(audio_ready){
				//	debug("BET: %d Dur: %d\n", (int)af_frame->best_effort_timestamp, /*(af_frame->nb_samples*1000)/af_frame->sample_rate*/af_frame->pkt_duration);
				//}
				if(ap_ptr < 10){
					avcodec_decode_audio4(audio_codec_context, af_frame, &audio_ready, &packet);
				}
				av_copy_packet(&audio_packets[ap_ptr++], &packet);
			}
		}
		av_seek_frame(fmt_context, video_stream, 0, AVSEEK_FLAG_BACKWARD); //Seek to the beginning to loop
	}	while(input.loop && input.multi_frame && !kill_thread);
	debug("Dropped through loop\n");
	if(input.multi_frame){
		stop_NBTV_gen(); //If it is a video and loop is set to off, stop the generation at the end
	}
	return 0;
}
/*Sets up some reasonable input values*/
void setup_input(struct _input *i){
	input.rotation_angle = 0;
	if(updated_fn || i->w/2+i->x > i->Xres || i->h2/2+i->y > i->Yres){
		if(input.Xres > input.Yres*input.input_aspect){
			i->h2 = i->Yres;
			i->w  = i->h2*input.input_aspect;
		} else {
			i->w = i->Xres;
			i->h2 = round(i->Xres/input.input_aspect);
		}
		i->x = i->Xres/2;
		i->y = i->Yres/2;
	}
	if(!i->h1 || i->h1 > i->h2){
		i->h1 = i->h2 * .8;
	}
}
/*Tries to load the input file with SDL_Image. Returns a pointer to the surface on success or NULL on failure*/
SDL_Surface *try_SDL_image(struct _input *i){
	SDL_Surface *img;
	char *ext = strrchr(i->filename, '.');
	if(!ext) return NULL;
	if(!strcmpi(ext, ".gif")) return NULL; //Do NOT load GIFs with SDL because if it is an animated GIF, it will only show the first frame
	img = IMG_Load(i->filename);
	if(!img){
		warn("SDL_Image failed to open %s\n", i->filename);
		return NULL; 
	}
	SDL_Surface *ret = SDL_ConvertSurface(img, screen->format, 0);
	SDL_FreeSurface(img);
	return ret;
}
int load_input_file(struct _input *i){
	debug("Load input file(%s)\n", i->filename);
	if(*i->filename==0){
		y_u_no_error();
		return 0;
	}
	fflush(stdout);
	//Try to load it with SDL_Image first. It's faster and lighter than FFMPEG but doesn't work for animated images/video
	image = try_SDL_image(i);
	if(image){ //Good! SDL_Image worked
		i->Xres = image->w;
		i->Yres = image->h;
		i->sxres = image->w;
		i->syres = image->h;
		i->XRR = i->YRR = 1.0;
		i->prescaled = 0; //No prescaling. The other layers will handle it
		setup_input(i);
		gui_update_input();
		updated_fn = 0;
		format_changed = 1;
		return 1;
	}
	if(avformat_open_input(&fmt_context, i->filename, NULL, NULL)){ //Otherwise try to use libavformat and libavcodec
		error("libavformat failed to open %s\n", i->filename);
		return 0;
	}
	if(avformat_find_stream_info(fmt_context, NULL)){
		error("find_stream_info() failed\n");
		return 0;
	}
	input_timestamp = 0;
	av_dump_format(fmt_context, 0, i->filename, 0);
	video_stream = find_video_stream(fmt_context);
	if(video_stream == -1){
		error("Failed to find video stream\n");
		return 0;
	}
	audio_stream = find_audio_stream(fmt_context);
	if(audio_stream == -1){
		info("No audio stream found");
	} else {
		debug("Audio stream %d", audio_stream);
		audio_codec_context = fmt_context->streams[audio_stream]->codec;
		audio_decoder = avcodec_find_decoder(audio_codec_context->codec_id);
		af_frame = av_frame_alloc();
		if(avcodec_open2(audio_codec_context, audio_decoder, NULL)){
			warn("Audio codec open failed");
		}
		resampler = swr_alloc_set_opts(resampler, AV_CH_LAYOUT_MONO, AV_SAMPLE_FMT_FLT, output.sample_rate, audio_codec_context->channel_layout, audio_codec_context->sample_fmt, audio_codec_context->sample_rate, 0, NULL);
		swr_init(resampler);
	}
	codec_context = fmt_context->streams[video_stream]->codec;
	decoder = avcodec_find_decoder(codec_context->codec_id);
	if(avcodec_open2(codec_context, decoder, NULL)){
		error("Avcodec open failed\n");
		return 0;
	}
	i->length = fmt_context->duration/1000.0; //Convert the duration to milliseconds
	int w,h;
	//out = malloc(320 * 240 * 3);
	w = codec_context->width;
	h = codec_context->height;
	/*Find the biggest width and height that is a multple while preserving aspect*/
	while(w*2 < (screen->w/2)+48 && h*2 < screen->h){
		w*=2;
		h*=2;
	}
	while(w > (screen->w/2)+48 || h > screen->h){
		w/=1.25;
		h/=1.25;
	}
	/*Align it to 4 bytes (libswscale seems to have issues if it is not aligned*/
	if(w & 3){w += (4 - (w&3));}
	if(h & 3){h += (4 - (h&3));}
	i->Xres = codec_context->width; //Actuall resolution
	i->Yres = codec_context->height;
	i->sxres = w; //Resolution of prescaled image
	i->syres = h;
	i->XRR = (double)w/(double)i->Xres; //Ratio between scaled and unscaled
	i->YRR = (double)h/(double)i->Yres;
	i->prescaled = 1; //It is scaled already so it doesn't need to be scaled again before blitting to the display
	debug("w = %d h = %d Xres = %d Yres = %d XRR = %0.1f YRR = %0.1f\n", w, h, i->Xres, i->Yres, i->XRR, i->YRR);
	scaler = sws_getCachedContext(scaler, codec_context->width, codec_context->height, codec_context->pix_fmt, w, h, AV_PIX_FMT_BGR24, SWS_POINT, NULL, NULL, NULL);
	av_frame = av_frame_alloc();
	//av_frame_scaled = av_frame_alloc();
	/*It is a still image. Fudge a delay value for one frame*/
	if(!codec_context->framerate.num){
		delay_value = 1000;
		i->multi_frame = 0;
	} else {
		/*It's a video. Compute the frame delay from libavcodec's weird-ass "rationals"*/
		//delay_value = codec_context->framerate.den*1000/codec_context->framerate.num;
		delay_value = (double)((double)((double)codec_context->time_base.num*1000.0)/(double)codec_context->time_base.den);
		debug("%ld/%ld sizeof(long)=%d\n", codec_context->time_base.num, codec_context->time_base.den, sizeof(long));
		delay_value *= codec_context->ticks_per_frame;
		i->multi_frame = 1;
		debug("TPF = %d\n", codec_context->ticks_per_frame);
	}
	debug("Framerate=%d/%d (%0.2f) dv=%d\nw=%d h=%d\n", codec_context->framerate.num, codec_context->framerate.den, (double)codec_context->framerate.num/(double)codec_context->framerate.den, delay_value,w,h);
	image = SDL_CreateRGBSurface(0, w, h, 24, 0xFF0000, 0x00FF00, 0x0000FF, 0);
	setup_input(i);
	kill_thread = 0;
	SDL_CreateThread(video_thread, NULL);
	gui_update_input();
	updated_fn = 0;
	format_changed = 1;
	return 1;
}
void video_stop(){
	kill_thread = 1;
	SDL_Delay(250);
	avcodec_close(codec_context);
	avformat_close_input(&fmt_context);
	av_frame_free(&av_frame);
	gui_update_input();
}
SDL_Surface *frame_window(struct _input *in, struct NBTV_format *f){
	//if(input.window_type) return NULL;
	if(image == NULL){return NULL;}
	static SDL_Surface *windowed;
	if(!windowed || format_changed){
		if(windowed){
			SDL_FreeSurface(windowed);
			windowed = NULL;
		}
		windowed = SDL_CreateRGBSurface(0, f->imgW, f->imgH, 24, 0xFF0000, 0x00FF00, 0x0000FF, 0);
		if(!windowed){
			return NULL;
		}
	}

	/*This is a bit of a hack. If the image is not prescaled, the windowing fucntions expect
	XRR and YRR to be 1.0 but other functions depend on them being set based on the screen scaling
	so we save the value here and temporarily set it to 1.0 before calling the windowing function and
	then restore the original value*/
	double tXRR,tYRR;
	if(!in->prescaled){
		tXRR = in->XRR;
		tYRR = in->YRR;
		in->XRR = 1.0;
		in->YRR = 1.0;
	}
		if(input.window_type == 0)rectangular_window(windowed, nbtv_fmt, image, &input);
		else if(input.window_type == 1)trapizoidal_window(windowed, nbtv_fmt, image, &input); 
	if(!in->prescaled){
		in->XRR = tXRR;
		in->YRR = tYRR;
	}
	return windowed;
}
	
	